@@ -1,5 +1,5 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * vim: set ts=8 sts=4 et sw=4 tw=99:
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+ * vim: set ts=8 sts=2 et sw=2 tw=80:
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -14,6 +14,7 @@ static const JS::Value JSVAL_TRUE =
     JS::Value::fromTagAndPayload(JSVAL_TAG_BOOLEAN, true);
 static const JS::Value JSVAL_VOID =
     JS::Value::fromTagAndPayload(JSVAL_TAG_UNDEFINED, 0);
+static const mozilla::Maybe<JS::Value> JSVAL_NOTHING;
 
 namespace JS {
 
@@ -25,5 +26,7 @@ const HandleValue TrueHandleValue =
     HandleValue::fromMarkedLocation(&JSVAL_TRUE);
 const HandleValue FalseHandleValue =
     HandleValue::fromMarkedLocation(&JSVAL_FALSE);
+const Handle<mozilla::Maybe<Value>> NothingHandleValue =
+    Handle<mozilla::Maybe<Value>>::fromMarkedLocation(&JSVAL_NOTHING);
 
 }  // namespace JS
