@@ -1,5 +1,5 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * vim: set ts=8 sts=4 et sw=4 tw=99:
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+ * vim: set ts=8 sts=2 et sw=2 tw=80:
  *
  * Copyright 2015 Mozilla Foundation
  *
@@ -27,7 +27,8 @@ namespace js {
 namespace wasm {
 
 // Return whether IonCompileFunction() can generate code on the current device.
-bool IonCanCompile();
+// Usually you do *not* want this, you want IonAvailable().
+MOZ_MUST_USE bool IonPlatformSupport();
 
 // Generates very fast code at the expense of compilation time.
 MOZ_MUST_USE bool IonCompileFunctions(const ModuleEnvironment& env,
