@@ -23,7 +23,6 @@
 #include "mozStorageStatementRow.h"
 #include "mozStorageStatement.h"
 #include "GeckoProfiler.h"
-#include "nsDOMClassInfo.h"
 
 #include "mozilla/Logging.h"
 #include "mozilla/Printf.h"
@@ -48,43 +47,43 @@ class StatementClassInfo : public nsIClassInfo {
   NS_DECL_ISUPPORTS_INHERITED
 
   NS_IMETHOD
-  GetInterfaces(uint32_t *_count, nsIID ***_array) override {
-    return NS_CI_INTERFACE_GETTER_NAME(Statement)(_count, _array);
+  GetInterfaces(nsTArray<nsIID>& _array) override {
+    return NS_CI_INTERFACE_GETTER_NAME(Statement)(_array);
   }
 
   NS_IMETHOD
-  GetScriptableHelper(nsIXPCScriptable **_helper) override {
+  GetScriptableHelper(nsIXPCScriptable** _helper) override {
     static StatementJSHelper sJSHelper;
     *_helper = &sJSHelper;
     return NS_OK;
   }
 
   NS_IMETHOD
-  GetContractID(nsACString &aContractID) override {
+  GetContractID(nsACString& aContractID) override {
     aContractID.SetIsVoid(true);
     return NS_OK;
   }
 
   NS_IMETHOD
-  GetClassDescription(nsACString &aDesc) override {
+  GetClassDescription(nsACString& aDesc) override {
     aDesc.SetIsVoid(true);
     return NS_OK;
   }
 
   NS_IMETHOD
-  GetClassID(nsCID **_id) override {
+  GetClassID(nsCID** _id) override {
     *_id = nullptr;
     return NS_OK;
   }
 
   NS_IMETHOD
-  GetFlags(uint32_t *_flags) override {
+  GetFlags(uint32_t* _flags) override {
     *_flags = 0;
     return NS_OK;
   }
 
   NS_IMETHOD
-  GetClassIDNoAlloc(nsCID *_cid) override { return NS_ERROR_NOT_AVAILABLE; }
+  GetClassIDNoAlloc(nsCID* _cid) override { return NS_ERROR_NOT_AVAILABLE; }
 };
 
 NS_IMETHODIMP_(MozExternalRefCountType) StatementClassInfo::AddRef() {
@@ -103,12 +102,14 @@ static StatementClassInfo sStatementClassInfo;
 Statement::Statement()
     : StorageBaseStatementInternal(),
       mDBStatement(nullptr),
+      mParamCount(0),
+      mResultColumnCount(0),
       mColumnNames(),
       mExecuting(false) {}
 
-nsresult Statement::initialize(Connection *aDBConnection,
-                               sqlite3 *aNativeConnection,
-                               const nsACString &aSQLStatement) {
+nsresult Statement::initialize(Connection* aDBConnection,
+                               sqlite3* aNativeConnection,
+                               const nsACString& aSQLStatement) {
   MOZ_ASSERT(aDBConnection, "No database connection given!");
   MOZ_ASSERT(aDBConnection->isConnectionReadyOnThisThread(),
              "Database connection should be valid");
@@ -136,9 +137,9 @@ nsresult Statement::initialize(Connection *aDBConnection,
   mResultColumnCount = ::sqlite3_column_count(mDBStatement);
   mColumnNames.Clear();
 
-  nsCString *columnNames = mColumnNames.AppendElements(mResultColumnCount);
+  nsCString* columnNames = mColumnNames.AppendElements(mResultColumnCount);
   for (uint32_t i = 0; i < mResultColumnCount; i++) {
-    const char *name = ::sqlite3_column_name(mDBStatement, i);
+    const char* name = ::sqlite3_column_name(mDBStatement, i);
     columnNames[i].Assign(name);
   }
 
@@ -147,7 +148,7 @@ nsresult Statement::initialize(Connection *aDBConnection,
   // escapeStringForLIKE instead of just trusting user input.  The idea to
   // check to see if they are binding a parameter after like instead of just
   // using a string.  We only do this in debug builds because it's expensive!
-  const nsCaseInsensitiveCStringComparator c;
+  auto c = nsCaseInsensitiveCStringComparator;
   nsACString::const_iterator start, end, e;
   aSQLStatement.BeginReading(start);
   aSQLStatement.EndReading(end);
@@ -181,7 +182,7 @@ nsresult Statement::initialize(Connection *aDBConnection,
   return NS_OK;
 }
 
-mozIStorageBindingParams *Statement::getParams() {
+mozIStorageBindingParams* Statement::getParams() {
   nsresult rv;
 
   // If we do not have an array object yet, make it.
@@ -190,7 +191,7 @@ mozIStorageBindingParams *Statement::getParams() {
     rv = NewBindingParamsArray(getter_AddRefs(array));
     NS_ENSURE_SUCCESS(rv, nullptr);
 
-    mParamsArray = static_cast<BindingParamsArray *>(array.get());
+    mParamsArray = static_cast<BindingParamsArray*>(array.get());
   }
 
   // If there isn't already any rows added, we'll have to add one to use.
@@ -229,7 +230,7 @@ NS_INTERFACE_MAP_BEGIN(Statement)
   NS_INTERFACE_MAP_ENTRY(mozIStorageValueArray)
   NS_INTERFACE_MAP_ENTRY(mozilla::storage::StorageBaseStatementInternal)
   if (aIID.Equals(NS_GET_IID(nsIClassInfo))) {
-    foundInterface = static_cast<nsIClassInfo *>(&sStatementClassInfo);
+    foundInterface = static_cast<nsIClassInfo*>(&sStatementClassInfo);
   } else
     NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, mozIStorageStatement)
 NS_INTERFACE_MAP_END
@@ -237,9 +238,9 @@ NS_INTERFACE_MAP_END
 ////////////////////////////////////////////////////////////////////////////////
 //// StorageBaseStatementInternal
 
-Connection *Statement::getOwner() { return mDBConnection; }
+Connection* Statement::getOwner() { return mDBConnection; }
 
-int Statement::getAsyncStatement(sqlite3_stmt **_stmt) {
+int Statement::getAsyncStatement(sqlite3_stmt** _stmt) {
   // If we have no statement, we shouldn't be calling this method!
   NS_ASSERTION(mDBStatement != nullptr, "We have no statement to clone!");
 
@@ -261,10 +262,10 @@ int Statement::getAsyncStatement(sqlite3_stmt **_stmt) {
   return SQLITE_OK;
 }
 
-nsresult Statement::getAsynchronousStatementData(StatementData &_data) {
+nsresult Statement::getAsynchronousStatementData(StatementData& _data) {
   if (!mDBStatement) return NS_ERROR_UNEXPECTED;
 
-  sqlite3_stmt *stmt;
+  sqlite3_stmt* stmt;
   int rc = getAsyncStatement(&stmt);
   if (rc != SQLITE_OK) return convertResultCode(rc);
 
@@ -274,7 +275,7 @@ nsresult Statement::getAsynchronousStatementData(StatementData &_data) {
 }
 
 already_AddRefed<mozIStorageBindingParams> Statement::newBindingParams(
-    mozIStorageBindingParamsArray *aOwner) {
+    mozIStorageBindingParamsArray* aOwner) {
   nsCOMPtr<mozIStorageBindingParams> params = new BindingParams(aOwner, this);
   return params.forget();
 }
@@ -286,7 +287,7 @@ already_AddRefed<mozIStorageBindingParams> Statement::newBindingParams(
 MIXIN_IMPL_STORAGEBASESTATEMENTINTERNAL(Statement, (void)0;)
 
 NS_IMETHODIMP
-Statement::Clone(mozIStorageStatement **_statement) {
+Statement::Clone(mozIStorageStatement** _statement) {
   RefPtr<Statement> statement(new Statement());
   NS_ENSURE_TRUE(statement, NS_ERROR_OUT_OF_MEMORY);
 
@@ -364,7 +365,7 @@ nsresult Statement::internalFinalize(bool aDestructing) {
 }
 
 NS_IMETHODIMP
-Statement::GetParameterCount(uint32_t *_parameterCount) {
+Statement::GetParameterCount(uint32_t* _parameterCount) {
   if (!mDBStatement) return NS_ERROR_NOT_INITIALIZED;
 
   *_parameterCount = mParamCount;
@@ -372,11 +373,11 @@ Statement::GetParameterCount(uint32_t *_parameterCount) {
 }
 
 NS_IMETHODIMP
-Statement::GetParameterName(uint32_t aParamIndex, nsACString &_name) {
+Statement::GetParameterName(uint32_t aParamIndex, nsACString& _name) {
   if (!mDBStatement) return NS_ERROR_NOT_INITIALIZED;
   ENSURE_INDEX_VALUE(aParamIndex, mParamCount);
 
-  const char *name =
+  const char* name =
       ::sqlite3_bind_parameter_name(mDBStatement, aParamIndex + 1);
   if (name == nullptr) {
     // this thing had no name, so fake one
@@ -391,7 +392,7 @@ Statement::GetParameterName(uint32_t aParamIndex, nsACString &_name) {
 }
 
 NS_IMETHODIMP
-Statement::GetParameterIndex(const nsACString &aName, uint32_t *_index) {
+Statement::GetParameterIndex(const nsACString& aName, uint32_t* _index) {
   if (!mDBStatement) return NS_ERROR_NOT_INITIALIZED;
 
   // We do not accept any forms of names other than ":name", but we need to add
@@ -408,7 +409,7 @@ Statement::GetParameterIndex(const nsACString &aName, uint32_t *_index) {
 }
 
 NS_IMETHODIMP
-Statement::GetColumnCount(uint32_t *_columnCount) {
+Statement::GetColumnCount(uint32_t* _columnCount) {
   if (!mDBStatement) return NS_ERROR_NOT_INITIALIZED;
 
   *_columnCount = mResultColumnCount;
@@ -416,18 +417,18 @@ Statement::GetColumnCount(uint32_t *_columnCount) {
 }
 
 NS_IMETHODIMP
-Statement::GetColumnName(uint32_t aColumnIndex, nsACString &_name) {
+Statement::GetColumnName(uint32_t aColumnIndex, nsACString& _name) {
   if (!mDBStatement) return NS_ERROR_NOT_INITIALIZED;
   ENSURE_INDEX_VALUE(aColumnIndex, mResultColumnCount);
 
-  const char *cname = ::sqlite3_column_name(mDBStatement, aColumnIndex);
+  const char* cname = ::sqlite3_column_name(mDBStatement, aColumnIndex);
   _name.Assign(nsDependentCString(cname));
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-Statement::GetColumnIndex(const nsACString &aName, uint32_t *_index) {
+Statement::GetColumnIndex(const nsACString& aName, uint32_t* _index) {
   if (!mDBStatement) return NS_ERROR_NOT_INITIALIZED;
 
   // Surprisingly enough, SQLite doesn't provide an API for this.  We have to
@@ -463,12 +464,12 @@ Statement::Reset() {
 }
 
 NS_IMETHODIMP
-Statement::BindParameters(mozIStorageBindingParamsArray *aParameters) {
+Statement::BindParameters(mozIStorageBindingParamsArray* aParameters) {
   NS_ENSURE_ARG_POINTER(aParameters);
 
   if (!mDBStatement) return NS_ERROR_NOT_INITIALIZED;
 
-  BindingParamsArray *array = static_cast<BindingParamsArray *>(aParameters);
+  BindingParamsArray* array = static_cast<BindingParamsArray*>(aParameters);
   if (array->getOwner() != this) return NS_ERROR_UNEXPECTED;
 
   if (array->length() == 0) return NS_ERROR_UNEXPECTED;
@@ -491,8 +492,8 @@ Statement::Execute() {
 }
 
 NS_IMETHODIMP
-Statement::ExecuteStep(bool *_moreResults) {
-  AUTO_PROFILER_LABEL("Statement::ExecuteStep", STORAGE);
+Statement::ExecuteStep(bool* _moreResults) {
+  AUTO_PROFILER_LABEL("Statement::ExecuteStep", OTHER);
 
   if (!mDBStatement) return NS_ERROR_NOT_INITIALIZED;
 
@@ -548,7 +549,7 @@ Statement::ExecuteStep(bool *_moreResults) {
 }
 
 NS_IMETHODIMP
-Statement::GetState(int32_t *_state) {
+Statement::GetState(int32_t* _state) {
   if (!mDBStatement)
     *_state = MOZ_STORAGE_STATEMENT_INVALID;
   else if (mExecuting)
@@ -563,13 +564,13 @@ Statement::GetState(int32_t *_state) {
 //// mozIStorageValueArray (now part of mozIStorageStatement too)
 
 NS_IMETHODIMP
-Statement::GetNumEntries(uint32_t *_length) {
+Statement::GetNumEntries(uint32_t* _length) {
   *_length = mResultColumnCount;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-Statement::GetTypeOfIndex(uint32_t aIndex, int32_t *_type) {
+Statement::GetTypeOfIndex(uint32_t aIndex, int32_t* _type) {
   if (!mDBStatement) return NS_ERROR_NOT_INITIALIZED;
 
   ENSURE_INDEX_VALUE(aIndex, mResultColumnCount);
@@ -601,7 +602,7 @@ Statement::GetTypeOfIndex(uint32_t aIndex, int32_t *_type) {
 }
 
 NS_IMETHODIMP
-Statement::GetInt32(uint32_t aIndex, int32_t *_value) {
+Statement::GetInt32(uint32_t aIndex, int32_t* _value) {
   if (!mDBStatement) return NS_ERROR_NOT_INITIALIZED;
 
   ENSURE_INDEX_VALUE(aIndex, mResultColumnCount);
@@ -613,7 +614,7 @@ Statement::GetInt32(uint32_t aIndex, int32_t *_value) {
 }
 
 NS_IMETHODIMP
-Statement::GetInt64(uint32_t aIndex, int64_t *_value) {
+Statement::GetInt64(uint32_t aIndex, int64_t* _value) {
   if (!mDBStatement) return NS_ERROR_NOT_INITIALIZED;
 
   ENSURE_INDEX_VALUE(aIndex, mResultColumnCount);
@@ -626,7 +627,7 @@ Statement::GetInt64(uint32_t aIndex, int64_t *_value) {
 }
 
 NS_IMETHODIMP
-Statement::GetDouble(uint32_t aIndex, double *_value) {
+Statement::GetDouble(uint32_t aIndex, double* _value) {
   if (!mDBStatement) return NS_ERROR_NOT_INITIALIZED;
 
   ENSURE_INDEX_VALUE(aIndex, mResultColumnCount);
@@ -639,7 +640,7 @@ Statement::GetDouble(uint32_t aIndex, double *_value) {
 }
 
 NS_IMETHODIMP
-Statement::GetUTF8String(uint32_t aIndex, nsACString &_value) {
+Statement::GetUTF8String(uint32_t aIndex, nsACString& _value) {
   // Get type of Index will check aIndex for us, so we don't have to.
   int32_t type;
   nsresult rv = GetTypeOfIndex(aIndex, &type);
@@ -649,7 +650,7 @@ Statement::GetUTF8String(uint32_t aIndex, nsACString &_value) {
     // string.
     _value.SetIsVoid(true);
   } else {
-    const char *value = reinterpret_cast<const char *>(
+    const char* value = reinterpret_cast<const char*>(
         ::sqlite3_column_text(mDBStatement, aIndex));
     _value.Assign(value, ::sqlite3_column_bytes(mDBStatement, aIndex));
   }
@@ -657,7 +658,7 @@ Statement::GetUTF8String(uint32_t aIndex, nsACString &_value) {
 }
 
 NS_IMETHODIMP
-Statement::GetString(uint32_t aIndex, nsAString &_value) {
+Statement::GetString(uint32_t aIndex, nsAString& _value) {
   // Get type of Index will check aIndex for us, so we don't have to.
   int32_t type;
   nsresult rv = GetTypeOfIndex(aIndex, &type);
@@ -667,15 +668,63 @@ Statement::GetString(uint32_t aIndex, nsAString &_value) {
     // string.
     _value.SetIsVoid(true);
   } else {
-    const char16_t *value = static_cast<const char16_t *>(
+    const char16_t* value = static_cast<const char16_t*>(
         ::sqlite3_column_text16(mDBStatement, aIndex));
-    _value.Assign(value, ::sqlite3_column_bytes16(mDBStatement, aIndex) / 2);
+    _value.Assign(value, ::sqlite3_column_bytes16(mDBStatement, aIndex) /
+                             sizeof(char16_t));
   }
   return NS_OK;
 }
 
 NS_IMETHODIMP
-Statement::GetBlob(uint32_t aIndex, uint32_t *_size, uint8_t **_blob) {
+Statement::GetVariant(uint32_t aIndex, nsIVariant** _value) {
+  if (!mDBStatement) {
+    return NS_ERROR_NOT_INITIALIZED;
+  }
+
+  ENSURE_INDEX_VALUE(aIndex, mResultColumnCount);
+
+  if (!mExecuting) {
+    return NS_ERROR_UNEXPECTED;
+  }
+
+  nsCOMPtr<nsIVariant> variant;
+  int type = ::sqlite3_column_type(mDBStatement, aIndex);
+  switch (type) {
+    case SQLITE_INTEGER:
+      variant =
+          new IntegerVariant(::sqlite3_column_int64(mDBStatement, aIndex));
+      break;
+    case SQLITE_FLOAT:
+      variant = new FloatVariant(::sqlite3_column_double(mDBStatement, aIndex));
+      break;
+    case SQLITE_TEXT: {
+      const char16_t* value = static_cast<const char16_t*>(
+          ::sqlite3_column_text16(mDBStatement, aIndex));
+      nsDependentString str(
+          value,
+          ::sqlite3_column_bytes16(mDBStatement, aIndex) / sizeof(char16_t));
+      variant = new TextVariant(str);
+      break;
+    }
+    case SQLITE_NULL:
+      variant = new NullVariant();
+      break;
+    case SQLITE_BLOB: {
+      int size = ::sqlite3_column_bytes(mDBStatement, aIndex);
+      const void* data = ::sqlite3_column_blob(mDBStatement, aIndex);
+      variant = new BlobVariant(std::pair<const void*, int>(data, size));
+      break;
+    }
+  }
+  NS_ENSURE_TRUE(variant, NS_ERROR_UNEXPECTED);
+
+  variant.forget(_value);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+Statement::GetBlob(uint32_t aIndex, uint32_t* _size, uint8_t** _blob) {
   if (!mDBStatement) return NS_ERROR_NOT_INITIALIZED;
 
   ENSURE_INDEX_VALUE(aIndex, mResultColumnCount);
@@ -683,58 +732,61 @@ Statement::GetBlob(uint32_t aIndex, uint32_t *_size, uint8_t **_blob) {
   if (!mExecuting) return NS_ERROR_UNEXPECTED;
 
   int size = ::sqlite3_column_bytes(mDBStatement, aIndex);
-  void *blob = nullptr;
+  void* blob = nullptr;
   if (size) {
-    blob = nsMemory::Clone(::sqlite3_column_blob(mDBStatement, aIndex), size);
-    NS_ENSURE_TRUE(blob, NS_ERROR_OUT_OF_MEMORY);
+    blob = moz_xmemdup(::sqlite3_column_blob(mDBStatement, aIndex), size);
   }
 
-  *_blob = static_cast<uint8_t *>(blob);
+  *_blob = static_cast<uint8_t*>(blob);
   *_size = size;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-Statement::GetBlobAsString(uint32_t aIndex, nsAString &aValue) {
+Statement::GetBlobAsString(uint32_t aIndex, nsAString& aValue) {
   return DoGetBlobAsString(this, aIndex, aValue);
 }
 
 NS_IMETHODIMP
-Statement::GetBlobAsUTF8String(uint32_t aIndex, nsACString &aValue) {
+Statement::GetBlobAsUTF8String(uint32_t aIndex, nsACString& aValue) {
   return DoGetBlobAsString(this, aIndex, aValue);
 }
 
 NS_IMETHODIMP
-Statement::GetSharedUTF8String(uint32_t aIndex, uint32_t *_length,
-                               const char **_value) {
-  if (_length) *_length = ::sqlite3_column_bytes(mDBStatement, aIndex);
-
-  *_value = reinterpret_cast<const char *>(
+Statement::GetSharedUTF8String(uint32_t aIndex, uint32_t* _byteLength,
+                               const char** _value) {
+  *_value = reinterpret_cast<const char*>(
       ::sqlite3_column_text(mDBStatement, aIndex));
+  if (_byteLength) {
+    *_byteLength = ::sqlite3_column_bytes(mDBStatement, aIndex);
+  }
   return NS_OK;
 }
 
 NS_IMETHODIMP
-Statement::GetSharedString(uint32_t aIndex, uint32_t *_length,
-                           const char16_t **_value) {
-  if (_length) *_length = ::sqlite3_column_bytes16(mDBStatement, aIndex);
-
-  *_value = static_cast<const char16_t *>(
+Statement::GetSharedString(uint32_t aIndex, uint32_t* _byteLength,
+                           const char16_t** _value) {
+  *_value = static_cast<const char16_t*>(
       ::sqlite3_column_text16(mDBStatement, aIndex));
+  if (_byteLength) {
+    *_byteLength = ::sqlite3_column_bytes16(mDBStatement, aIndex);
+  }
   return NS_OK;
 }
 
 NS_IMETHODIMP
-Statement::GetSharedBlob(uint32_t aIndex, uint32_t *_size,
-                         const uint8_t **_blob) {
-  *_size = ::sqlite3_column_bytes(mDBStatement, aIndex);
+Statement::GetSharedBlob(uint32_t aIndex, uint32_t* _byteLength,
+                         const uint8_t** _blob) {
   *_blob =
-      static_cast<const uint8_t *>(::sqlite3_column_blob(mDBStatement, aIndex));
+      static_cast<const uint8_t*>(::sqlite3_column_blob(mDBStatement, aIndex));
+  if (_byteLength) {
+    *_byteLength = ::sqlite3_column_bytes(mDBStatement, aIndex);
+  }
   return NS_OK;
 }
 
 NS_IMETHODIMP
-Statement::GetIsNull(uint32_t aIndex, bool *_isNull) {
+Statement::GetIsNull(uint32_t aIndex, bool* _isNull) {
   // Get type of Index will check aIndex for us, so we don't have to.
   int32_t type;
   nsresult rv = GetTypeOfIndex(aIndex, &type);
